package com.emarcotte.glfoo;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import com.sun.opengl.util.GLUT;

/**
 * GraphicsDrivers serve as the rendering mechanism between jogl and GlFoo
 * Renderers. A GraphicsDriver is typically added to a GLCanvas as a
 * GLEventListener where it can respond to events from the component etc.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 * 
 */
public interface GraphicsDriver {
	/**
	 * Sets what RenderingContext the GraphicsDriver will use to render the
	 * scene.
	 * 
	 * @param context
	 *            The context to use.
	 */
	public void setContext(RenderingContext context);

	/**
	 * While driving one of the Renderers, the GraphicsDriver holds onto a
	 * reference to the camera for various reasons.
	 * 
	 * TODO: It may be desirable to actually keep track of the Renderer which
	 * has the camera. It seems a bit cleaner.
	 * 
	 * @return The camera of the current Renderer.
	 */
	public Camera getCamera();

	/**
	 * The height of the GL drawing area.
	 * 
	 * @return Height of the drawing area.
	 */
	public int getHeight();

	/**
	 * The width of the GL drawing area.
	 * 
	 * @return Width of the drawing area.
	 */
	public int getWidth();

	/**
	 * The ratio of width to height as a double.
	 * 
	 * @return The width divided by height.
	 */
	public double getRatio();

	/**
	 * The GL context this GraphicsDriver is using to draw. It is preferable to
	 * call this method for operations that occur in Renderers and Renderables
	 * rather than caching the context. It may be possible to change the GL
	 * context which would result in some odd behavior.
	 * 
	 * @return The GL context that is being used to render.
	 */
	public GL getGl();

	/**
	 * For convenience the GraphicsDriver should provide a GLU which belongs to
	 * its current GL context.
	 * 
	 * @return The GraphicsDriver's GLU context.
	 */
	public GLU getGlu();

	/**
	 * For convenience the GraphicsDriver should provide a GLUT which belongs to
	 * its current GL context.
	 * 
	 * @return The GraphicsDriver's GLUT context.
	 */
	public GLUT getGlut();
}
