package com.emarcotte.glfoo;

/**
 * Renderables are objects which can be rendered by one or many of the
 * {@link Renderer}s.
 * 
 * @see com.emarcotte.glfoo.Renderer
 * @author Eugene Marcotte
 */
public interface Renderable {

	/**
	 * <code>draw()</code> is invoked by a particular renderer during their
	 * render loop. Its purpose is to primarily draw objects, though that
	 * doesn't matter.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void draw(GraphicsDriver driver);

	/**
	 * <code>init()</code> is invoked by a particular renderer during their
	 * render loop. Its purpose is to initialize details of renderables that
	 * require the opengl context. An example might be loading a texture,
	 * creating a text renderer.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void init(GraphicsDriver driver);
}
