package com.emarcotte.glfoo;

/**
 * Renderers provide the GraphicsDriver with something to do for each frame.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public interface Renderer {

	/**
	 * Get the Camera for this Renderer.
	 * 
	 * @return The Camera for this Renderer.
	 */
	public Camera getCamera();

	/**
	 * Do whatever this renderer needs to do on a per-frame basis.
	 * 
	 * An example would be the GraphicsRenderer/DefaultRenderer which simply
	 * render whatever renderables they have been provided.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void render(GraphicsDriver driver);

	/**
	 * Initializes the Renderer. This should really only happen one time. It
	 * *may* happen twice if this Renderer is added to two different Rendering
	 * contexts however.
	 * 
	 * This method is designed primarily to allow the Renderer to do
	 * initialization that requires an opengl context and not to replace a
	 * constructor.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void initialize(GraphicsDriver driver);

	/**
	 * Per-Renderer drawing/setup. This should be used for whole-scene setup
	 * things such as setting up projections or post-processing effects.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void renderBefore(GraphicsDriver driver);

	/**
	 * Post-Renderer drawing/cleanup. This should be used for whole-scene things
	 * such as post-processing effects.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void renderAfter(GraphicsDriver driver);
}