package com.emarcotte.glfoo;

import java.util.List;

/**
 * GlFoo users need to define a RenderingContext which, for the most part,
 * defines what is going to be Rendered. To do this, it needs to define a list
 * of Renderers.
 * 
 * The primary is intended to be something like this:
 * <ol>
 * <li> implement this interface
 * <li> during whatever initialization routines:
 * <ol>
 * <li> add Renderers
 * <li> add listeners/event handlers
 * </ol>
 * <li>tell the ApplicationContext about the new context.
 * </ol>
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public interface RenderingContext {

	/**
	 * One of the first things GlFoo does when drawing a frame is check to see
	 * if the rendering context has been activated. If it has not, it will call
	 * this activate method.
	 */
	public void activate();

	/**
	 * When GlFoo swaps rendering contexts, this method is called on the one
	 * that is no longer the current context.
	 */
	public void deactivate();

	/**
	 * The Renderers for the context.
	 * 
	 * @return The Renderers for the context.
	 */
	public List<Renderer> getRenderers();
}
