package com.emarcotte.glfoo.implementation;

import com.emarcotte.glfoo.ApplicationContext;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.MainWindow;
import com.emarcotte.glfoo.RenderingContext;

/**
 * The ApplicationContext is the core of the graphical portion of GlFoo. It
 * houses the window, opengl driver, opengl/mouse event handler and the
 * app-specific data.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class DefaultApplicationContext implements ApplicationContext {
	/**
	 * The main window the context is driving.
	 */
	private MainWindow mainWindow;

	/**
	 * The graphics driver for this context.
	 */
	private GraphicsDriver graphicsDriver;

	/**
	 * The current RenderingContext
	 */
	private RenderingContext currentRenderingContext;

	/**
	 * Get the GraphicsDriver that GlFoo is using currently.
	 * 
	 * @return The GraphicsDriver that is currently running.
	 */
	public GraphicsDriver getGraphicsDriver() {
		return graphicsDriver;
	}

	/**
	 * Set the GraphicsDriver that GlFoo should use.
	 * 
	 * XXX: This has not been tested at all -- calling thing anywhere after
	 * {@link #run()} may be very bad.
	 * 
	 * @param graphicsDriver
	 */
	public void setGraphicsDriver(GraphicsDriver graphicsDriver) {
		this.graphicsDriver = graphicsDriver;
	}

	public MainWindow getMainWindow() {
		return mainWindow;
	}

	public void setMainWindow(MainWindow mainWindow) {
		this.mainWindow = mainWindow;
	}

	public RenderingContext getCurrentRenderingContext() {
		return currentRenderingContext;
	}

	public void setCurrentRenderingContext(RenderingContext newContext) {
		this.currentRenderingContext = newContext;
		graphicsDriver.setContext(newContext);
	}

	public void run() {
		mainWindow.start();
	}
}
