package com.emarcotte.glfoo.implementation;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.Camera;
import com.emarcotte.glfoo.GraphicsDriver;

/**
 * Cameras control the viewing of a scene. They are set by controling position,
 * angle, and perspective.
 * 
 * @author Eugene Marcotte
 */
public class DefaultCamera implements Camera {
	/**
	 * The perspective angle of the camera.
	 */
	private double perspective;

	/**
	 * The x component of the camera's position.
	 */
	private double x;

	/**
	 * The x component of the camera's angle.
	 */
	private double xAngle;

	/**
	 * The y component of the camera's position.
	 */
	private double y;

	/**
	 * The y component of the camera's angle.
	 */
	private double yAngle;

	/**
	 * The z component of the camera's position.
	 */
	private double z;

	/**
	 * The z component of the camera's angle.
	 */
	private double zAngle;

	private double near;

	private double far;

	/**
	 * Construct a camera with default settings.
	 * 
	 * Default perspective = 45 Default depth = 5 near = .01 far = 1000
	 */
	public DefaultCamera() {
		setPerspective(45);
		setZ(5);
		setNear(.01);
		setFar(1000);
	}

	/**
	 * Apply the perspective to the view. This camera does not switch to the
	 * projection matrix -- consider yourself warned.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */
	public void applyPerspective(GraphicsDriver driver) {
		driver.getGlu().gluPerspective(perspective, driver.getRatio(), near,
				far);
	}

	/**
	 * Apply positions: rotates by the 3 component angles, then translates to
	 * the specified position.
	 * 
	 * @param driver
	 *            The GraphicsDriver that is currently running.
	 */

	public void applyPosition(GraphicsDriver driver) {
		GL gl = driver.getGl();
		gl.glTranslated(-x, -y, -z);
		gl.glRotated(xAngle, 1, 0, 0);
		gl.glRotated(yAngle, 0, 1, 0);
	}

	public double getPerspective() {
		return perspective;
	}

	public void setX(double x) {
		this.x = x;
	}

	public void setY(double y) {
		this.y = y;
	}

	public void setZ(double z) {
		this.z = z;
	}

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}

	public double getZ() {
		return z;
	}

	public void addX(double d) {
		x += d;
	}

	public void addY(double d) {
		y += d;
	}

	public void addZ(double d) {
		z += d;
	}

	public void setXAngle(double angle) {
		xAngle = angle;
	}

	public void setYAngle(double angle) {
		yAngle = angle;
	}

	public void setZAngle(double angle) {
		zAngle = angle;
	}

	public void addXAngle(double d) {
		xAngle += d;
	}

	public void addYAngle(double d) {
		yAngle += d;
	}

	public void addZAngle(double d) {
		zAngle += d;
	}

	public double getXAngle() {
		return xAngle;
	}

	public double getYAngle() {
		return yAngle;
	}

	public double getZAngle() {
		return zAngle;
	}

	public void setPerspective(double perspective) {
		this.perspective = perspective;
	}

	public void setNear(double n) {
		near = n;
	}

	public void setFar(double f) {
		far = f;
	}
}
