package com.emarcotte.glfoo.implementation;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;

import com.emarcotte.glfoo.Camera;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Renderer;
import com.emarcotte.glfoo.RenderingContext;
import com.sun.opengl.util.GLUT;

/**
 * Default implementation of the {@link GraphicsDriver} for GlFoo. This
 * implementation explicitly states that it implements GLEventListener due to
 * the fact that one day graphics drivers may not actually extend event
 * listeners (we shall see).
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class DefaultGraphicsDriver implements GLEventListener, GraphicsDriver {
	/**
	 * The current {@link RenderingContext} that is going to be used to draw the
	 * scene.
	 */
	private RenderingContext currentContext;
	/**
	 * Before switching rendering contexts, the new context is stored here. When
	 * swapping the de-activate is called on the old one and activate on this
	 * one. This will generally be null, meaning there is no new context.
	 */
	private RenderingContext newContext;
	/**
	 * The camera
	 */
	private Camera camera;

	private int height;

	private int width;

	private double ratio;

	private GLUT glut;

	private GLU glu;

	private GL gl;

	public DefaultGraphicsDriver() {
	}

	protected void swapContexts() {
		if (newContext != null) {
			if (currentContext != null) {
				currentContext.deactivate();
			}
			newContext.activate();
			currentContext = newContext;
			newContext = null;
		}
	}

	@Override
	public void display(GLAutoDrawable glDrawable) {
		gl = glDrawable.getGL();

		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();

		swapContexts();

		for (Renderer r : currentContext.getRenderers()) {
			camera = r.getCamera();
			r.initialize(this);
			r.renderBefore(this);
			r.render(this);
			r.renderAfter(this);
		}
	}

	@Override
	public void displayChanged(GLAutoDrawable glDrawable, boolean a, boolean b) {

	}

	@Override
	public void init(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		glut = new GLUT();
		glu = new GLU();

		gl.glDepthFunc(GL.GL_LEQUAL);
		gl.glCullFace(GL.GL_BACK);
		gl.glEnable(GL.GL_BLEND);
		gl.glDisable(GL.GL_NORMALIZE);

		gl.glEnable(GL.GL_DEPTH_TEST);
	}

	@Override
	public void reshape(GLAutoDrawable glDrawable, int x, int y, int w, int h) {
		GL gl = glDrawable.getGL();
		height = h;
		width = w;
		ratio = (double) w / (double) h;
		if (height == 0) {
			height = 1;
		}

		gl.glViewport(0, 0, width, height);
	}

	@Override
	public void setContext(RenderingContext context) {
		newContext = context;
	}

	@Override
	public Camera getCamera() {
		return camera;
	}

	@Override
	public int getHeight() {
		return height;
	}

	@Override
	public int getWidth() {
		return width;
	}

	@Override
	public GL getGl() {
		return gl;
	}

	@Override
	public GLU getGlu() {
		return glu;
	}

	@Override
	public GLUT getGlut() {
		return glut;
	}

	@Override
	public double getRatio() {
		return ratio;
	}
}
