package com.emarcotte.glfoo.implementation;

import com.emarcotte.glfoo.Camera;
import com.emarcotte.glfoo.GraphicsDriver;

/**
 * This {@link Camera} sets up an orthographic projection. The configuration is
 * such that the current {@link GraphicsDriver}'s width and height are the
 * dimensions of the plane at z = 0. The bottom left corner of the viewport will
 * be at 0,0 and the top right at width,height.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class OrthographicCamera implements Camera {
	/**
	 * Default near value.
	 */
	public static final double DEFAULT_NEAR = -1000.0;
	/**
	 * Default far value.
	 */
	public static final double DEFAULT_FAR = 1000.0;

	/**
	 * Near cutoff.
	 */
	private double near;
	/**
	 * Far cutoff
	 */
	private double far;

	/**
	 * Default constructor applies near = -10.0 and far = 10.0
	 */
	public OrthographicCamera() {
		setNear(DEFAULT_NEAR);
		setFar(DEFAULT_FAR);
	}

	@Override
	public void applyPerspective(GraphicsDriver driver) {
		driver.getGl().glOrtho(0, driver.getWidth(), 0, driver.getHeight(),
				near, far);
	}

	public double getNear() {
		return near;
	}

	public void setNear(double near) {
		this.near = near;
	}

	public double getFar() {
		return far;
	}

	public void setFar(double far) {
		this.far = far;
	}

	@Override
	public void applyPosition(GraphicsDriver driver) {

	}

}
