package com.emarcotte.glfoo.math;

public class Vector3 {
	public double i;
	public double j;
	public double k;

	public Vector3() {
		this(0, 0, 0);
	}

	public Vector3(double i, double j, double k) {
		setI(i);
		setJ(j);
		setK(k);
	}

	public Vector3(Vector3 vector) {
		this(vector.getI(), vector.getJ(), vector.getK());
	}

	public void copy(Vector3 v) {
		setI(v.getI());
		setJ(v.getJ());
		setK(v.getK());
	}

	public double getI() {
		return i;
	}

	public double getJ() {
		return j;
	}

	public double getK() {
		return k;
	}

	public void setI(double d) {
		i = d;
	}

	public void setJ(double d) {
		j = d;
	}

	public void setK(double d) {
		k = d;
	}

	public void setVector(Vector3 v1) {
		setI(v1.getI());
		setJ(v1.getJ());
		setK(v1.getK());
	}

	public void addI(double d) {
		i += d;
	}

	public void addJ(double d) {
		j += d;
	}

	public void addK(double d) {
		k += d;
	}

	public Vector3 subtract(Vector3 v1) {
		Vector3 out = new Vector3();
		subtract(v1, out);
		return out;
	}

	public void subtract(Vector3 v1, Vector3 out) {
		out.setI(getI() - v1.getI());
		out.setJ(getJ() - v1.getJ());
		out.setK(getK() - v1.getK());
	}

	public Vector3 add(Vector3 v1) {
		Vector3 out = new Vector3();
		add(v1, out);
		return out;
	}

	public void add(Vector3 v1, Vector3 out) {
		out.setI(getI() + v1.getI());
		out.setJ(getJ() + v1.getJ());
		out.setK(getK() + v1.getK());
	}

	public void addMe(Vector3 v1) {
		setI(getI() + v1.getI());
		setJ(getJ() + v1.getJ());
		setK(getK() + v1.getK());
	}

	public void zero() {
		setI(0);
		setJ(0);
		setK(0);
	}

	public double[] toArray() {
		return new double[] { i, j, k };
	}

	public Vector3 crossProduct(Vector3 v2) {
		Vector3 cross = new Vector3();
		crossProduct(v2, cross);
		return cross;
	}

	public void crossProduct(Vector3 v2, Vector3 cross) {
		cross.setI(getJ() * v2.getK() - getK() * v2.getJ());
		cross.setJ(getK() * v2.getI() - getI() * v2.getK());
		cross.setK(getI() * v2.getJ() - getJ() * v2.getI());
	}

	public double dot(Vector3 v2) {
		return getI() * v2.getI() + getJ() * v2.getJ() + getK() * v2.getK();
	}

	// *joe - rewrote a little to use other functions
	public void normalize() {
		// double length = Math.sqrt(Math.pow(getI(), 2) + Math.pow(getJ(), 2)
		// + Math.pow(getK(), 2));
		double length = magnitude();

		if (length == 0.0 || length == 1) {
			return;
		}

		// setI(getI() / length);
		// setJ(getJ() / length);
		// setK(getK() / length);

		divide(length, this);

	}

	public double magnitude() {
		return Math.sqrt((getI() * getI()) + (getJ() * getJ())
				+ (getK() * getK()));
	}

	public void divide(Vector3 v1, Vector3 out) {
		// Put some error handling shit here sometime.
		out.setI(getI() / v1.getI());
		out.setJ(getJ() / v1.getJ());
		out.setK(getK() / v1.getK());
	}

	public void divide(double scalar, Vector3 out) {
		// Put some error handling shit here sometime.
		out.setI(getI() / scalar);
		out.setJ(getJ() / scalar);
		out.setK(getK() / scalar);

	}

	public Vector3 multiply(double scalar) {
		Vector3 v2 = new Vector3(this);
		v2.multiply(scalar, v2);
		return v2;
	}

	public void multiply(Vector3 v1, Vector3 out) {
		out.setI(getI() * v1.getI());
		out.setJ(getJ() * v1.getJ());
		out.setK(getK() * v1.getK());
	}

	public void multiply(double scalar, Vector3 out) {
		out.setI(getI() * scalar);
		out.setJ(getJ() * scalar);
		out.setK(getK() * scalar);
	}

	public void multiplyMe(double scalar) {
		multiply(scalar, this);
	}

	public void linearInterpolate(Vector3 one, Vector3 two, double factor) {
		setI(one.getI() + factor * (two.getI() - one.getI()));
		setJ(one.getJ() + factor * (two.getJ() - one.getJ()));
		setK(one.getK() + factor * (two.getK() - one.getK()));
	}

	public String toString() {
		return "{" + getI() + ", " + getJ() + ", " + getK() + "}";
	}
}
