package com.emarcotte.glfoo.utils;

import java.util.HashMap;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.GraphicsDriver;

/**
 * TODO: This ugly mess of crap is not generic. It should basically implement
 * the sort of interface for frame buffers and let you attach arbitrary numbers
 * of textures render buffers etc.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class GLFrameBufferObject {
	static HashMap<Integer, String> fboStatusMap;

	static {
		fboStatusMap = new HashMap<Integer, String>();
		fboStatusMap.put(GL.GL_FRAMEBUFFER_COMPLETE_EXT, "Complete");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT,
				"Incomplete attachment");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT,
				"Incomplete missing attachment");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT,
				"Incomplete dimensions");

		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT,
				"Incomplete formats");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT,
				"Incomplete draw buffer");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT,
				"Incomplete read buffer");
		fboStatusMap.put(GL.GL_FRAMEBUFFER_UNSUPPORTED_EXT,
				"FBO is unsupported");
	}
	private int fboId;
	private int texId;
	private int depthBuffer;
	private GL gl;

	public GLFrameBufferObject(GraphicsDriver driver) {
		gl = driver.getGl();
		int[] buff = new int[1];

		// setup the FBO
		gl.glGenFramebuffersEXT(1, buff, 0);
		fboId = buff[0];
		gl.glBindFramebufferEXT(GL.GL_FRAMEBUFFER_EXT, fboId);

		// setup the texture to render into
		gl.glGenTextures(1, buff, 0);

		texId = buff[0];
		gl.glBindTexture(GL.GL_TEXTURE_2D, texId);
		gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA8, driver.getWidth(),
				driver.getHeight(), 0, GL.GL_RGBA, GL.GL_INT, null);
		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER,
				GL.GL_LINEAR);
		gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER,
				GL.GL_LINEAR);

		// Bind texture to FBO
		gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
				GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_2D, texId, 0);

		// depth buffer
		gl.glGenRenderbuffersEXT(1, buff, 0);
		depthBuffer = buff[0];
		gl.glBindRenderbufferEXT(GL.GL_RENDERBUFFER_EXT, depthBuffer);
		gl.glRenderbufferStorageEXT(GL.GL_RENDERBUFFER_EXT,
				GL.GL_DEPTH_COMPONENT24, driver.getWidth(), driver.getHeight());

		
		gl.glFramebufferRenderbufferEXT(GL.GL_FRAMEBUFFER_EXT,
						GL.GL_DEPTH_ATTACHMENT_EXT,
						GL.GL_RENDERBUFFER_EXT,
						depthBuffer);

		// Check status
		int status = gl.glCheckFramebufferStatusEXT(GL.GL_FRAMEBUFFER_EXT);
		if (status != GL.GL_FRAMEBUFFER_COMPLETE_EXT) {
			System.out.println(getStatus(status));
		}

		gl.glBindFramebufferEXT(GL.GL_FRAMEBUFFER_EXT, 0);
		gl.glBindTexture(GL.GL_TEXTURE_2D, 0);
	}

	private static String getStatus(int status) {
		return fboStatusMap.get(status);
	}

	public void bind() {
		gl.glBindFramebufferEXT(GL.GL_FRAMEBUFFER_EXT, fboId);
	}

	public void unbind() {
		gl.glBindFramebufferEXT(GL.GL_FRAMEBUFFER_EXT, 0);
	}

	public int getTextureId() {
		return texId;
	}
}
