package com.emarcotte.glfoo.utils;

import java.nio.IntBuffer;

import javax.media.opengl.GL;

import com.sun.opengl.util.BufferUtil;

public class GLShaderProgram {
	private int programId;

	private GL gl;

	private GLShaderProgram(GL gl, int id) {
		this.gl = gl;
		this.programId = id;
	}

	public static GLShaderProgram createProgram(GL gl) {
		int id = gl.glCreateProgram();
		if (id == 0) {
			return null;
		} else {
			return new GLShaderProgram(gl, id);
		}
	}

	public void add(GLShader shader) {
		gl.glAttachShader(programId, shader.getShaderId());
		int glErr = gl.glGetError();
		if (glErr != GL.GL_NO_ERROR)
			System.out.println("Error attaching");
	}

	public void link() {
		gl.glLinkProgram(programId);
		checkLog();
	}

	private void checkLog() {
		IntBuffer status = BufferUtil.newIntBuffer(1);
		IntBuffer validationStatus = BufferUtil.newIntBuffer(1);

		gl.glGetProgramiv(programId, GL.GL_LINK_STATUS, status);

		if (status.get() == GL.GL_FALSE) {
			System.out.println("Program " + programId + " failed to link");
		}

		IntBuffer length = BufferUtil.newIntBuffer(1);

		gl.glGetProgramiv(programId, GL.GL_INFO_LOG_LENGTH, length);

		int l = length.get();
		byte[] log = new byte[l];
		gl.glGetProgramInfoLog(programId, l, null, 0, log, 0);

		String sLog = new String(log).trim();
		if (sLog.length() > 0) {
			System.out.println("Program information log: ");
			System.out.println(sLog);
		}

		gl.glValidateProgram(programId);

		gl.glGetProgramiv(programId, GL.GL_VALIDATE_STATUS, validationStatus);

		if (validationStatus.get() == GL.GL_FALSE) {
			System.out.println("Program is not valid.");
		}
	}

	public void setUniform3f(String name, float f1, float f2, float f3) {
		int id = this.lookupUniform(name);

		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}

		gl.glUniform3f(id, f1, f2, f3);
	}

	public void setUniform1fv(String name, float[] v) {
		int id = this.lookupUniform(name);

		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}

		gl.glUniform1fv(id, v.length, v, 0);

	}

	public void setUniformb(String name, boolean b) {
		int id = this.lookupUniform(name);
		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}
		if (b) {
			gl.glUniform1i(id, 1);
		} else {
			gl.glUniform1i(id, 0);
		}
	}

	public void setUniformf(String name, float x) {
		int id = this.lookupUniform(name);
		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}
		gl.glUniform1f(id, x);
	}

	public void setUniformi(String name, int i) {
		int id = this.lookupUniform(name);
		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}
		gl.glUniform1i(id, i);
	}

	public void setAttribute3f(String name, float x, float y, float z) {
		int id = this.lookupAttribute(name);
		if (id == -1) {
			System.out.format(Messages.getString("Cant_Find_Variable"), name,
					programId);
			return;
		}
		gl.glVertexAttrib3d(id, x, y, z);
	}

	public void enable() {
		gl.glUseProgram(programId);
	}

	public void disable() {
		gl.glUseProgram(0);
	}

	public int getProgramId() {
		return programId;
	}

	public int lookupUniform(String name) {
		return gl.glGetUniformLocation(programId, name);
	}

	public int lookupAttribute(String name) {
		return gl.glGetAttribLocation(programId, name);
	}

}
