package com.emarcotte.glfoo.utils;

import java.nio.IntBuffer;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.GraphicsDriver;
import com.sun.opengl.util.BufferUtil;

public class IndexedVertexBufferObject extends VertexBufferObject {
	protected IntBuffer indices;
	protected int indexBuffer;

	public IndexedVertexBufferObject(int[] indices) {
		this.indices = IntBuffer.wrap(indices);
	}

	@Override
	public void draw(GraphicsDriver driver) {
		GL gl = driver.getGl();

		gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, positionBuffer);
		gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, verts.capacity()
				* BufferUtil.SIZEOF_DOUBLE, verts, GL.GL_STREAM_DRAW);
		gl.glVertexPointer(3, GL.GL_DOUBLE, 0, 0);

		gl.glBindBufferARB(GL.GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		gl.glBufferData(GL.GL_ELEMENT_ARRAY_BUFFER, BufferUtil.SIZEOF_INT
				* indices.capacity(), indices, GL.GL_STREAM_DRAW);

		if (texCoords != null) {
			gl.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, texCoordBuffer);
			gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, texCoords.capacity()
					* BufferUtil.SIZEOF_DOUBLE, texCoords, GL.GL_STREAM_DRAW);
			gl.glTexCoordPointer(2, GL.GL_DOUBLE, 0, 0);
		}

		if (normals != null) {
			gl.glEnableClientState(GL.GL_NORMAL_ARRAY);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, normalBuffer);
			gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, normals.capacity()
					* BufferUtil.SIZEOF_DOUBLE, normals, GL.GL_STREAM_DRAW);
			gl.glNormalPointer(GL.GL_DOUBLE, 0, 0);
		}

		gl.glDrawElements(faceMode, indices.capacity(), GL.GL_UNSIGNED_INT, 0);

		gl.glDisableClientState(GL.GL_NORMAL_ARRAY);
		gl.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY);
		gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
	}

	@Override
	public void init(GraphicsDriver driver) {
		super.init(driver);
		indexBuffer = GLUtil.genBuffer(driver.getGl());
	}
}
