package com.emarcotte.glfoo.demos;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.ApplicationContext;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.implementation.ApplicationContextProvider;
import com.emarcotte.glfoo.implementation.DefaultCamera;
import com.emarcotte.glfoo.implementation.DefaultRenderingContext;
import com.emarcotte.glfoo.implementation.DefaultSelectable;
import com.emarcotte.glfoo.implementation.GraphicRenderer;
import com.emarcotte.glfoo.implementation.SelectableEventHandler;
import com.emarcotte.glfoo.implementation.SelectionRenderer;

/**
 * Simple demo with spinning cube that handles mouse events.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class CubeDemo2 extends DefaultRenderingContext {

	protected static class Cube implements SelectableEventHandler, Renderable {
		double rotation = 0;
		int id;

		@Override
		public void draw(GraphicsDriver driver) {
			GL gl = driver.getGl();

			gl.glRotated(rotation, .3, 1, 0);
			rotation += 0.1;
			gl.glBegin(GL.GL_QUADS);
			gl.glColor3d(1, .5, .5);
			gl.glVertex3d(1, -1, -1);
			gl.glVertex3d(1, -1, 1);
			gl.glVertex3d(1, 1, 1);
			gl.glVertex3d(1, 1, -1);
			gl.glColor3d(0, .5, .5);
			gl.glVertex3d(-1, -1, -1);
			gl.glVertex3d(-1, -1, 1);
			gl.glVertex3d(-1, 1, 1);
			gl.glVertex3d(-1, 1, -1);
			gl.glColor3d(.5, 1, .5);
			gl.glVertex3d(-1, 1, -1);
			gl.glVertex3d(-1, 1, 1);
			gl.glVertex3d(1, 1, 1);
			gl.glVertex3d(1, 1, -1);
			gl.glColor3d(.5, 0, .5);
			gl.glVertex3d(-1, -1, -1);
			gl.glVertex3d(-1, -1, 1);
			gl.glVertex3d(1, -1, 1);
			gl.glVertex3d(1, -1, -1);
			gl.glColor3d(.5, .5, 1);
			gl.glVertex3d(-1, -1, 1);
			gl.glVertex3d(1, -1, 1);
			gl.glVertex3d(1, 1, 1);
			gl.glVertex3d(-1, 1, 1);
			gl.glColor3d(.5, .5, 0);
			gl.glVertex3d(-1, -1, -1);
			gl.glVertex3d(1, -1, -1);
			gl.glVertex3d(1, 1, -1);
			gl.glVertex3d(-1, 1, -1);
			gl.glEnd();
		}

		@Override
		public void init(GraphicsDriver driver) {

		}

		@Override
		public void onClick() {
			System.out.println("click");
		}

		@Override
		public void onDrag(double i, double j) {
			System.out.println("drag " + i + "  " + j);
		}

		@Override
		public void onEnter() {
			System.out.println("enter");
		}

		@Override
		public void onLeave() {
			System.out.println("leave");
		}

		@Override
		public void onPressed() {
			System.out.println("pressed");
		}

		@Override
		public void onReleased() {
			System.out.println("released");
		}
	}

	public CubeDemo2(ApplicationContext context) {
		Cube c = new Cube();
		DefaultCamera camera = new DefaultCamera();
		GraphicRenderer gr = new GraphicRenderer(camera);
		gr.addRenderable(c);
		addRenderer(gr);
		SelectionRenderer sr = new SelectionRenderer(camera);
		DefaultSelectable s = new DefaultSelectable(c);
		s.addEventHandler(c);
		sr.registerSelectable(s);
		addRenderer(sr);
		context.getMainWindow().addMouseHandler(sr);

	}

	public static void main(String[] args) {
		ApplicationContext context = ApplicationContextProvider.getInstance()
				.getContext();
		context.setCurrentRenderingContext(new CubeDemo2(context));
		context.run();
	}

}
