package com.emarcotte.glfoo.demos;

import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.nio.DoubleBuffer;
import java.text.AttributedCharacterIterator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.ApplicationContext;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.Renderer;
import com.emarcotte.glfoo.RenderingContext;
import com.emarcotte.glfoo.implementation.ApplicationContextProvider;
import com.emarcotte.glfoo.implementation.DefaultCamera;
import com.emarcotte.glfoo.implementation.DefaultMouseHandler;
import com.emarcotte.glfoo.implementation.GraphicRenderer;
import com.emarcotte.glfoo.utils.GLUtil;
import com.sun.opengl.util.BufferUtil;
import com.sun.opengl.util.j2d.TextRenderer;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

public class TestVBORenderable implements Renderable, KeyListener {

	private int positionBuffer;
	private int texCoordBuffer;
	private int normalBuffer;
	DoubleBuffer verts;
	DoubleBuffer texCoords;
	DoubleBuffer normals;
	private boolean vbo;
	private Texture t;
	private int faceCount;
	private TextRenderer text;

	public TestVBORenderable(int numFaces) {

		this.faceCount = numFaces;
		verts = DoubleBuffer.allocate(3 * numFaces * 6);
		texCoords = DoubleBuffer.allocate(2 * numFaces * 6);
		normals = DoubleBuffer.allocate(3 * numFaces * 6);
		verts.rewind();
		for (int i = 0; i < numFaces; i++) {
			verts.put(-1);
			verts.put(-numFaces / 2.0 + i);
			verts.put(0);
			verts.put(1);
			verts.put(-numFaces / 2.0 + i);
			verts.put(0);
			verts.put(-1);
			verts.put(-numFaces / 2.0 + i + 1);
			verts.put(0);
			verts.put(-1);
			verts.put(-numFaces / 2.0 + i + 1);
			verts.put(0);
			verts.put(1);
			verts.put(-numFaces / 2.0 + i);
			verts.put(0);
			verts.put(1);
			verts.put(-numFaces / 2.0 + i + 1);
			verts.put(0);
			texCoords.put(0);
			texCoords.put(0);
			texCoords.put(1);
			texCoords.put(0);
			texCoords.put(0);
			texCoords.put(1);
			texCoords.put(0);
			texCoords.put(1);
			texCoords.put(1);
			texCoords.put(0);
			texCoords.put(1);
			texCoords.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
			normals.put(0);
			normals.put(0);
			normals.put(1);
		}
		vbo = true;
	}

	@Override
	public void draw(GraphicsDriver driver) {
		GL gl = driver.getGl();
		t.bind();
		gl.glPushMatrix();
		gl.glTranslated(3 * faceCount - 5, 0, 0);

		text.begin3DRendering();
		text.setColor(Color.BLACK);
		text.draw3D((vbo ? "VBO" : "glVertex(s)"), -.05f, faceCount, 0, .01f);
		text.end3DRendering();
		gl.glColor3d(1, 1, 1);
		if (vbo) {
			gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
			gl.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY);
			gl.glEnableClientState(GL.GL_NORMAL_ARRAY);

			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, texCoordBuffer);
			gl.glTexCoordPointer(2, GL.GL_DOUBLE, 0, 0);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, normalBuffer);
			gl.glNormalPointer(GL.GL_DOUBLE, 0, 0);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, positionBuffer);
			gl.glVertexPointer(3, GL.GL_DOUBLE, 0, 0);

			gl.glDrawArrays(GL.GL_TRIANGLES, 0, verts.capacity() / 3);
			gl.glDisableClientState(GL.GL_NORMAL_ARRAY);
			gl.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY);
			gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
		} else {

			gl.glBegin(GL.GL_TRIANGLES);
			for (int i = 0; i < verts.capacity() / 3; i++) {
				gl.glTexCoord2d(texCoords.get(i * 2), texCoords.get(i * 2 + 1));
				gl.glNormal3d(normals.get(i * 3), normals.get(i * 3 + 1),
						normals.get(i * 3 + 2));
				gl.glVertex3d(verts.get(i * 3), verts.get(i * 3 + 1), verts
						.get(i * 3 + 2));
			}
			gl.glEnd();
		}
		gl.glPopMatrix();
		gl.glBindTexture(t.getTarget(), 0);
	}

	@Override
	public void init(GraphicsDriver driver) {
		GL gl = driver.getGl();

		text = new TextRenderer(
				Font
						.getFont(new HashMap<AttributedCharacterIterator.Attribute, String>()).deriveFont(32));

		BufferedImage image = new BufferedImage(512, 512, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = image.createGraphics();
		GradientPaint p = new GradientPaint(256, 256, Color.yellow, 0, 0, Color.red);
		g.setPaint(p);
		Ellipse2D.Double circle = new Ellipse2D.Double(256, 256, 500, 500);
		g.fill(circle);
		t = TextureIO.newTexture(image, false);
		t.enable();

		int[] buffers = GLUtil.genBuffers(gl, 3);
		positionBuffer = buffers[0];
		texCoordBuffer = buffers[1];
		normalBuffer = buffers[2];

		verts.rewind();
		texCoords.rewind();
		normals.rewind();
		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, positionBuffer);
		gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, verts.capacity()
				* BufferUtil.SIZEOF_DOUBLE, verts, GL.GL_STREAM_DRAW);

		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, texCoordBuffer);
		gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, texCoords.capacity()
				* BufferUtil.SIZEOF_DOUBLE, texCoords, GL.GL_STREAM_DRAW);

		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, normalBuffer);
		gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, normals.capacity()
				* BufferUtil.SIZEOF_DOUBLE, normals, GL.GL_STREAM_DRAW);

	}

	@Override
	public void keyPressed(KeyEvent e) {

	}

	@Override
	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_V) {
			vbo = !vbo;
		}

	}

	@Override
	public void keyTyped(KeyEvent e) {

	}

	private static class RC implements RenderingContext {
		List<Renderer> renderers;

		public RC(ApplicationContext c) {
			renderers = new LinkedList<Renderer>();
			DefaultCamera camera = new DefaultCamera();
			GraphicRenderer renderer = new GraphicRenderer(camera);
			renderers.add(renderer);

			c.getMainWindow().addMouseHandler(new DefaultMouseHandler(camera));
			
			for(int i = 4 ; i > 0; i --) {
				TestVBORenderable t = new TestVBORenderable(i);
				renderer.addRenderable(t);
				c.getMainWindow().addKeyListener(t);
			}
		}

		@Override
		public void activate() {

		}

		@Override
		public void deactivate() {

		}

		@Override
		public List<Renderer> getRenderers() {
			return renderers;
		}

	}

	public static void main(String args[]) {
		ApplicationContext c = ApplicationContextProvider.getInstance()
				.getContext();
		c.setCurrentRenderingContext(new RC(c));
		c.run();
	}
}
