package com.emarcotte.glfoo.math;

import org.junit.Assert;
import org.junit.Test;

public class QuaternionTest {

	public static void quatCheck(Quaternion q, double x, double y, double z,
			double w) {
		Assert.assertNotNull(q);
		Assert.assertEquals(x, q.getX());
		Assert.assertEquals(y, q.getY());
		Assert.assertEquals(z, q.getZ());
		Assert.assertEquals(w, q.getW());
	}

	@Test
	public void testQuaternion() {
		Quaternion q1 = new Quaternion();
		quatCheck(q1, 0.0, 0.0, 0.0, 1);

		Quaternion q2 = new Quaternion(1, 2, 3);
		quatCheck(q2, 1.0, 2, 3, 0);

		Quaternion q3 = new Quaternion(-1, 2, 0);
		quatCheck(q3, -1, 2, 0.0, 0);

		Quaternion q4 = new Quaternion(1, 2, 3, 4);
		quatCheck(q4, 1.0, 2, 3, 4);

		Quaternion q5 = new Quaternion(0.0, 0.0, 0.0, 1);
		quatCheck(q5, 0.0, 0.0, 0.0, 1);

		Quaternion q7 = new Quaternion(q5);
		quatCheck(q7, 0.0, 0.0, 0.0, 1);
		quatCheck(q5, 0.0, 0.0, 0.0, 1);
		Quaternion q8 = new Quaternion(q7);
		quatCheck(q8, 0.0, 0.0, 0.0, 1);
		quatCheck(q7, 0.0, 0.0, 0.0, 1);
		quatCheck(q5, 0.0, 0.0, 0.0, 1);

		Quaternion q9 = new Quaternion(new Vector3(1, 0, 0), 90.0);
		quatCheck(q9, Math.sqrt(2) / 2, 0, 0, Math.sqrt(2) / 2);

		Quaternion q10 = new Quaternion(new Vector3(-1, -1, 1), 3);
		quatCheck(q10, -0.015113268, -0.015113268, 0.015113268, .999657325);

	}

	@Test(expected = NullPointerException.class)
	public void testQuaternionThrow() {
		new Quaternion(null);
	}

	@Test(expected = NullPointerException.class)
	public void testQuaternionThrow2() {
		new Quaternion(null, 90);
	}

	@Test
	public void testNormalize() {
		Quaternion q1 = new Quaternion();
		quatCheck(q1, 0.0, 0.0, 0.0, 1);
		q1.normalize();
		quatCheck(q1, 0.0, 0.0, 0.0, 1);

		Quaternion q2 = new Quaternion(1, 2, 3);
		quatCheck(q2, 1.0, 2, 3, 0);
		q2.normalize();
		quatCheck(q2, 1 / 36, 2 / 36, 3 / 36, 0);

		Quaternion q3 = new Quaternion(-1, 2, 0);
		quatCheck(q3, -1, 2, 0.0, 0);
		q3.normalize();
		quatCheck(q3, -1 / 9, 2 / 9, 0.0, 0);

		Quaternion q4 = new Quaternion(1, 2, 3, 4);
		quatCheck(q4, 1.0, 2, 3, 4);
		q4.normalize();
		quatCheck(q4, 1 / 30, 2 / 30, 3 / 30, 4 / 30);

		Quaternion q5 = new Quaternion(1.0, 0.0, 0.0, 0.0);
		quatCheck(q5, 1.0, 0.0, 0.0, 0);
		q5.normalize();
		quatCheck(q5, 1, 0, 0, 0);

		Quaternion q7 = new Quaternion(q5);
		q7.normalize();
		quatCheck(q7, 1, 0, 0, 0);
	}

	@Test
	public void testMultiplicationQuaternion() {
		// 0 0 0 1 x 0 0 0 1 = 0 0 0 1
		Quaternion q1 = new Quaternion(0, 0, 0, 1);
		Quaternion q2 = new Quaternion(0, 0, 0, 1);
		Quaternion result = q1.multiplication(q1);
		quatCheck(result, 0, 0, 0, 1);
		result = q1.multiplication(q1);
		quatCheck(result, 0, 0, 0, 1);

		// 0 0 0 -1 x 0 0 0 1 = 0 0 0 -1
		q1 = new Quaternion(0, 0, 0, -1);
		q2 = new Quaternion(0, 0, 0, 1);
		result = q1.multiplication(q2);
		quatCheck(result, 0, 0, 0, -1);

		// 2 0 0 1 x 0 0 0 -1 = -2 0 0 -1
		q1 = new Quaternion(2, 0, 0, 1);
		q2 = new Quaternion(0, 0, 0, -1);
		result = q1.multiplication(q2);
		quatCheck(result, -2, 0, 0, -1);

		// 2 0 -1 1 x 0 3 0 -1 = 1 3 7 -1
		q1 = new Quaternion(2, 0, -1, 1);
		q2 = new Quaternion(0, 3, 0, -1);
		result = q1.multiplication(q2);
		quatCheck(result, 1, 3, 7, -1);

		// 2 -2 3 1.5x 0 3 0 -1 = -11 6.5 3 4.5
		q1 = new Quaternion(2, -2, 3, 1.5);
		q2 = new Quaternion(0, 3, 0, -1);
		result = q1.multiplication(q2);
		quatCheck(result, -11, 6.5, 3, 4.5);

		// 2 -2 3 1.5 x 4.5 3 2 -1 = -8.25 16 15 -10.5
		q1 = new Quaternion(2, -2, 3, 1.5);
		q2 = new Quaternion(4.5, 3, 2, -1);
		result = q1.multiplication(q2);
		quatCheck(result, -8.25, 16, 15, -10.5);
	}

	@Test
	public void testMultiplicationQuaternionQuaternion() {
		// 0 0 0 1 x 0 0 0 1 = 0 0 0 1
		Quaternion result = new Quaternion();
		Quaternion q1 = new Quaternion(0, 0, 0, 1);
		Quaternion q2 = new Quaternion(0, 0, 0, 1);
		q1.multiplication(q2, result);
		quatCheck(result, 0, 0, 0, 1);
		q1.multiplication(q1, result);
		quatCheck(result, 0, 0, 0, 1);

		// 0 0 0 -1 x 0 0 0 1 = 0 0 0 -1
		q1 = new Quaternion(0, 0, 0, -1);
		q2 = new Quaternion(0, 0, 0, 1);
		q1.multiplication(q2, result);
		quatCheck(result, 0, 0, 0, -1);

		// 2 0 0 1 x 0 0 0 -1 = -2 0 0 -1
		q1 = new Quaternion(2, 0, 0, 1);
		q2 = new Quaternion(0, 0, 0, -1);
		q1.multiplication(q2, result);
		quatCheck(result, -2, 0, 0, -1);

		// 2 0 -1 1 x 0 3 0 -1 = 1 3 7 -1
		q1 = new Quaternion(2, 0, -1, 1);
		q2 = new Quaternion(0, 3, 0, -1);
		q1.multiplication(q2, result);
		quatCheck(result, 1, 3, 7, -1);

		// 2 -2 3 1.5x 0 3 0 -1 = -11 6.5 3 4.5
		q1 = new Quaternion(2, -2, 3, 1.5);
		q2 = new Quaternion(0, 3, 0, -1);
		q1.multiplication(q2, result);
		quatCheck(result, -11, 6.5, 3, 4.5);

		// 2 -2 3 1.5 x 4.5 3 2 -1 = -8.25 16 15 -10.5
		q1 = new Quaternion(2, -2, 3, 1.5);
		q2 = new Quaternion(4.5, 3, 2, -1);
		q1.multiplication(q2, result);
		quatCheck(result, -8.25, 16, 15, -10.5);
	}

	@Test
	public void testMultiplicationVector3() {
		// 1, 0, 0, 1 * 1, 1, 1 = 1, 0, 2, -1
		Vector3 v = new Vector3(1, 1, 1);
		Quaternion q = new Quaternion(1, 0, 0, 1);
		Quaternion result = q.multiplication(v);
		quatCheck(result, 1, 0, 2, -1);

		// 1, -5, 7, -3 * 0, 3, 2 = -31, -11, -3, 1
		q = new Quaternion(1, -5, 7, -3);
		v = new Vector3(0, 3, 2);
		result = q.multiplication(v);
		quatCheck(result, -31, -11, -3, 1);

		// more test cases needed
	}

	@Test
	public void testMultiplicationVector3Quaternion() {
		// 1, 0, 0, 1 * 1, 1, 1 = 1, 0, 2, -1
		Vector3 v = new Vector3(1, 1, 1);
		Quaternion q = new Quaternion(1, 0, 0, 1);
		Quaternion result = new Quaternion();
		q.multiplication(v, result);
		quatCheck(result, 1, 0, 2, -1);

		// 1, -5, 7, -3 * 0, 3, 2 = -31, -11, -3, 1
		q = new Quaternion(1, -5, 7, -3);
		v = new Vector3(0, 3, 2);
		q.multiplication(v, result);
		quatCheck(result, -31, -11, -3, 1);

		// more test cases needed
	}

	@Test
	public void testConjugate() {
		Quaternion q = new Quaternion(0, 0, 0, 1);
		quatCheck(q.conjugate(), 0, 0, 0, 1);
		q = new Quaternion(1, 2, 3, 4);
		quatCheck(q.conjugate(), -1, -2, -3, 4);
		q = new Quaternion(-1, 2, 4, 1);
		quatCheck(q.conjugate(), 1, -2, -4, 1);
		q = new Quaternion(2, -1, -3, 5);
		quatCheck(q.conjugate(), -2, 1, 3, 5);
	}

	@Test
	public void testConjugateQuaternion() {
		Quaternion q = new Quaternion(0, 0, 0, 1);
		Quaternion result = new Quaternion();
		q.conjugate(result);
		quatCheck(result, 0, 0, 0, 1);
		q = new Quaternion(1, 2, 3, 4);
		q.conjugate(result);
		quatCheck(result, -1, -2, -3, 4);
		q = new Quaternion(-1, 2, 4, 1);
		q.conjugate(result);
		quatCheck(result, 1, -2, -4, 1);
		q = new Quaternion(2, -1, -3, 5);
		q.conjugate(result);
		quatCheck(result, -2, 1, 3, 5);
	}

	public static void vecCheck(Vector3 v, double x, double y, double z) {
		Assert.assertNotNull(v);
		Assert.assertEquals(x, v.getI());
		Assert.assertEquals(y, v.getJ());
		Assert.assertEquals(z, v.getK());
	}

	@Test
	public void testRotatePointVector() {
		// simple 90 degree
		Quaternion q = new Quaternion(new Vector3(1, 0, 0), 90);
		Vector3 v = new Vector3(0, 1, 0);
		Vector3 result = q.rotatePoint(v);
		vecCheck(result, 0, 0, 1);

		// simple 90 degree
		v.setJ(-1);
		result = q.rotatePoint(v);
		vecCheck(result, 0, 0, -1);

		// no rotation
		q = new Quaternion(new Vector3(0, 1, 0), 90);
		v.setJ(1);
		result = q.rotatePoint(v);
		vecCheck(result, 0, 1, 0);

		// random axis
		q = new Quaternion(new Vector3(-1, -1, 1), 100 / 33);
		v = new Vector3(.3, -16, 3);
		result = q.rotatePoint(v);
		vecCheck(result, .6838566132569639, -15.88690173396646, 3.49695489290);
	}

	@Test
	public void testRotatePointVectorVector() {
		// simple 90 degree
		Quaternion q = new Quaternion(new Vector3(1, 0, 0), 90);
		Vector3 v = new Vector3(0, 1, 0);
		Vector3 result = new Vector3();
		q.rotatePoint(v, result);
		vecCheck(result, 0, 0, 1);

		// simple 90 degree
		v.setJ(-1);
		q.rotatePoint(v, result);
		vecCheck(result, 0, 0, -1);

		// no rotation
		q = new Quaternion(new Vector3(0, 1, 0), 90);
		v.setJ(1);
		q.rotatePoint(v, result);
		vecCheck(result, 0, 1, 0);

		// random axis
		q = new Quaternion(new Vector3(-1, -1, 1), 100 / 33);
		v = new Vector3(.3, -16, 3);
		q.rotatePoint(v, result);
		vecCheck(result, .6838566132569639, -15.88690173396646, 3.49695489290);
	}

	@Test
	public void testDotProduct() {
		Quaternion q1 = new Quaternion(0, 0, 0, 0);
		Quaternion q2 = new Quaternion(1, 0, 0, 0);
		Assert.assertEquals(0, q1.dotProduct(q1));
		Assert.assertEquals(0, q1.dotProduct(q2));
		Assert.assertEquals(1, q2.dotProduct(q2));

		q1 = new Quaternion(1, 0, 3, 0);
		Assert.assertEquals(1, q1.dotProduct(q2));
		Assert.assertEquals(1, q2.dotProduct(q1));

		q1 = new Quaternion(3, 4, 1, -10);
		q2 = new Quaternion(-1, 5.3, 10, 5);
		Assert.assertEquals(28.2, q1.dotProduct(q2));
		Assert.assertEquals(28.2, q2.dotProduct(q1));
		Assert.assertEquals(26, q1.dotProduct(q1));
		Assert.assertEquals(129.09, q2.dotProduct(q2));
	}

	@Test
	public void testCopy() {
		Quaternion q = new Quaternion(1, 2, 3, 4);
		Quaternion n = new Quaternion();
		n.copy(q);
		quatCheck(n, 1, 2, 3, 4);
		q = new Quaternion(-1, 14.6, 13.0894, 02.3);
		n.copy(q);
		quatCheck(n, -1, 14.6, 13.0894, 02.3);
	}

	@Test
	public void testSlerp() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testNlerp() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testSetFromEulerAxisAngle() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testSetX() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testSetY() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testSetZ() {
		Assert.fail("Not yet implemented");
	}

	@Test
	public void testSetW() {
		Assert.fail("Not yet implemented");
	}
}
