package com.emarcotte.testrio;

import java.util.List;

import com.emarcotte.glfoo.math.Vector3;

/**
 * MD5 ModelLoader UVCoordinate. Deals with weights and texture mapping
 * primarily.
 * 
 * @author Eugene Marcotte
 */

public class MD5Vertex {
	private int index;
	private int firstWeight;
	private int numWeights;
	private Vector3 wv;

	/**
	 * Create a vertex.
	 */
	public MD5Vertex(int index, int weightIndex, int weightElem) {
		this.firstWeight = weightIndex;
		this.numWeights = weightElem;
		this.index = index;
		wv = new Vector3();
	}

	public void recalculatePosition(MD5Mesh m, List<MD5Weight> weights,
			MD5Frame joints) {

		Vector3 normal = m.getNormal(index);
		Vector3 rotation = null;

		if (normal != null) {
			rotation = new Vector3(normal);
		}

		double x = 0, y = 0, z = 0;

		// calculate final vertex to draw with weights
		for (int i = 0; i < numWeights; ++i) {
			MD5Weight w = weights.get(firstWeight + i);
			MD5Joint j = joints.getJoint(w.getJoint());

			// calculate transformed vertex for this weight
			j.getOrientation().rotatePoint(w.getPosition(), wv);

			if (rotation != null) {
				j.getOrientation().rotatePoint(rotation, rotation);
			}

			// the sum of all weight->bias should be 1.0
			Vector3 jointPos = j.getPos();

			x += ((jointPos.getI() + wv.getI()) * w.getBias());
			y += ((jointPos.getJ() + wv.getJ()) * w.getBias());
			z += ((jointPos.getK() + wv.getK()) * w.getBias());

		}
		m.setVertData(index, x, y, z);

		if (rotation != null) {
			rotation.normalize();
			m.setNormalData(index, rotation);
		}
	}

	public int getId() {
		return index;
	}
}
