package modelview;

import java.awt.Frame;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLException;
import javax.media.opengl.glu.GLU;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;

import com.emarcotte.glfoo.utils.GLShader;
import com.emarcotte.glfoo.utils.GLShaderProgram;
import com.emarcotte.glfoo.utils.Light2;
import com.emarcotte.glfoo.utils.ResourceHandler;
import com.emarcotte.testrio.MD5Loader;
import com.emarcotte.testrio.MD5Model;
import com.sun.opengl.util.Animator;
import com.sun.opengl.util.GLUT;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

/**
 * TODO: Javadocs
 * 
 * @author Eugene Marcotte
 */
public class Modelview implements KeyListener, MouseMotionListener,
		WindowListener, GLEventListener {

	int mouseOldY;

	int mouseOldX;

	private int xrot;

	private int yrot;

	private double zslide;

	GLUT glut;

	GLU glu;

	private double z = -100;

	private GLCapabilities caps;
	private GLShaderProgram program;
	/**
	 * File extension for mesh files.
	 */
	public static final String MD5MESH_EXTENSION = ".md5mesh";

	/**
	 * List of models for rendering.
	 */
	private LinkedList<MD5Model> models;

	/**
	 * Window where things are displayed.
	 */
	private Frame window;

	/**
	 * Animator for the window.
	 */
	private Animator animator;

	/**
	 * Handle mouse drag events.
	 * 
	 * @param e
	 *            event info.
	 */
	public void mouseDragged(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			zslide = (e.getY() - mouseOldY);
		}

		if (SwingUtilities.isLeftMouseButton(e)) {
			yrot += e.getX() - mouseOldX;
			xrot += e.getY() - mouseOldY;
		}
		mouseOldY = e.getY();
		mouseOldX = e.getX();
	}

	/**
	 * Handle mouse movement events.
	 * 
	 * @param e
	 *            event info.
	 */
	public void mouseMoved(MouseEvent e) {
		mouseOldY = e.getY();
		mouseOldX = e.getX();
	}

	/**
	 * @param e
	 */
	public void windowDeactivated(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowActivated(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowIconified(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowDeiconified(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowOpened(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowClosed(WindowEvent e) {

	}

	/**
	 * @param e
	 */
	public void windowClosing(WindowEvent e) {
		animator.stop();
		window.setVisible(false);
		window.dispose();
	}

	/**
	 * @param event
	 */

	public void keyReleased(KeyEvent event) {
		if (event.getKeyCode() == KeyEvent.VK_ESCAPE) {
			windowClosing(null);
		} else if (event.getKeyCode() == KeyEvent.VK_UP) {
			MD5Model m = models.getLast();
			m.nextAnim();
		}
	}

	/**
	 * @param event
	 */

	public void keyTyped(KeyEvent event) {

	}

	/**
	 * @param event
	 */

	public void keyPressed(KeyEvent event) {

	}

	double x;

	/**
	 * Draw things.
	 * 
	 * @param glDrawable
	 *            does stuff
	 */
	public void display(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();

		// TextureManager.getInstance().uploadTextures(gl);

		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();
		gl.glClearColor(0, 0, 0, 0);
		gl.glColor3d(1, 1, 1);

		z += zslide;
		zslide = 0;

		gl.glTranslated(0, -50, z);
		gl.glRotated(xrot, 1, 0, 0);
		gl.glRotated(yrot, 0, 1, 0);
		gl.glPushMatrix();

		gl.glActiveTexture(GL.GL_TEXTURE0);
		normal.bind();
		gl.glActiveTexture(GL.GL_TEXTURE1);
		regular.bind();

		program.enable();
		for (MD5Model m : models) {
			m.draw(gl);
			gl.glTranslated(50, 0, 0);
		}
		program.disable();
		gl.glBindTexture(regular.getTarget(), 0);
		gl.glActiveTexture(0);
		gl.glBindTexture(normal.getTarget(), 0);
		try {
			Thread.sleep(10);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private Texture normal;
	private Texture regular;

	/**
	 * Setup GL.
	 * 
	 * @param glDrawable
	 */

	public void init(GLAutoDrawable glDrawable) {
		GL gl = glDrawable.getGL();
		glut = new GLUT();
		glu = new GLU();
		gl.glShadeModel(GL.GL_SMOOTH);
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepth(1.0);
		gl.glDepthFunc(GL.GL_LEQUAL);
		gl.glCullFace(GL.GL_BACK);
		// gl.glEnable(GL.GL_BLEND);
		gl.glDisable(GL.GL_NORMALIZE);
		try {
			normal = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("bark0/bark0DOT3.jpg"), true,
					TextureIO.JPG);
			regular = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("bark0/bark0.jpg"), true,
					TextureIO.JPG);
		} catch (GLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);

		Light2 l = new Light2();
		l.setDirectional(false);
		l.setPosition(0, 0, 100);
		l.setAmbientColor(.5f, .5f, .5f);
		l.setDiffuseColor(1, 1, 1);
		l.applyLight(GL.GL_LIGHT0, gl);
		gl.glEnable(GL.GL_LIGHTING);
		gl.glEnable(GL.GL_LIGHT1);
		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glEnable(GL.GL_DEPTH_TEST);

		program = GLShaderProgram.createProgram(gl);

		GLShader goochVertexShader = GLShader.load(gl, glu,
				"shaders/outline_vert.glsl", GL.GL_VERTEX_SHADER);

		GLShader goochFragShader = GLShader.load(gl, glu,
				"shaders/outline_frag.glsl", GL.GL_FRAGMENT_SHADER);

		program.add(goochVertexShader);
		program.add(goochFragShader);
		program.link();
		program.enable();
		program.setUniformi("colorMap", 1);
		program.setUniformi("normalMap", 0);
		program.disable();
	}

	/**
	 * Resize the GL stuff when the window is changed.
	 * 
	 * @param glDrawable
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 */

	public void reshape(GLAutoDrawable glDrawable, int x, int y, int w, int h) {
		GL gl = glDrawable.getGL();

		if (h == 0) {
			h = 1;
		}

		gl.glViewport(0, 0, w, h);

		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(45.0, w / h, 0.1, 10000.0);

		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	/**
	 * Setup the application.
	 */

	public Modelview() {
		models = new LinkedList<MD5Model>();

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});

	}

	public void createAndShowGUI() {
		MenuBar b = new MenuBar();
		Menu file = new Menu("File");
		MenuItem open = new MenuItem("Open");
		file.add(open);
		b.add(file);
		open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser f = new JFileChooser(".");

				f.setFileFilter(new MD5MeshFileFilter());
				int result = f.showOpenDialog(window);
				if (result == JFileChooser.APPROVE_OPTION) {
					File mesh = f.getSelectedFile();

					try {
						addMesh(mesh);
					} catch (FileNotFoundException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
			}
		});

		caps = new GLCapabilities();
		caps.setDoubleBuffered(true);
		caps.setHardwareAccelerated(true);

		GLCanvas canvas = new GLCanvas(caps);
		canvas.addGLEventListener(this);

		window = new Frame("Modelviewer");
		window.add(canvas);
		window.setMenuBar(b);

		// window.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		window.setSize(640, 480);
		window.setVisible(true);
		window.addWindowListener(this);

		animator = new Animator(canvas);
		animator.start();

		canvas.addKeyListener(this);
		canvas.addMouseMotionListener(this);
		canvas.requestFocus();

		/*
		 * canvas.setCursor( Toolkit.getDefaultToolkit().createCustomCursor( new
		 * BufferedImage( 1, 1, BufferedImage.TYPE_INT_ARGB ), new Point( 0, 0 ), "" ) );
		 */
	}

	/**
	 * @param gl
	 * @param one
	 * @param two
	 */

	public void displayChanged(GLAutoDrawable gl, boolean one, boolean two) {

	}

	/**
	 * Reads through a directory looking for MD5 meshes and animations to read.
	 * 
	 * @param mesh
	 *            to read
	 * @throws FileNotFoundException
	 */

	public void addMesh(File mesh) throws FileNotFoundException {
		File[] anims = mesh.getParentFile().listFiles(
				new MD5AnimationFileFilter());
		Map<String, InputStream> files = new HashMap<String, InputStream>();

		for (File f : anims) {

			files.put(f.getName(), new FileInputStream(f));
		}

		try {
			MD5Loader m = new MD5Loader();
			m.loadModel(new FileInputStream(mesh), files);
			models.add(m.getModel());
		} catch (FileNotFoundException e) {
			System.err.println(e.getMessage());
		}
	}

	/**
	 * Run the model viewer
	 * 
	 * @param args
	 *            the models to view
	 */

	public static void main(String args[]) {
		new Modelview();
	}
}
